"""Utility Functions for Supporting Other Modules"""
import requests
import json
import inspect
from servicepytan import URL_ROOT, AUTH_ROOT
from servicepytan.auth import get_auth_headers, get_tenant_id

def request_json(url, options={}, payload="", config_file='servicepytan_config.json', request_type="GET", json_payload=""):
  """Makes the request to the API and returns JSON

  Retrives JSON response from provided URL with a number of parameters to customize the request.

  Args:
      url: A string with the full URL request
      options: A dictionary defining the parameters to add to the url for filtering
      payload: A dictionary defining the data object to create or update
      config_file: A string for the file path to the configuration file.
      request_type: A string to define the REST endpoint type [GET, POST, PUT, PATCH, DEL].

  Returns:
      JSON Object

  Raises:
      TBD
  """
  headers = get_auth_headers(config_file)
  response = requests.request(request_type, url, data=payload, headers=headers, params=options, json=json_payload)
  return json.loads(response.text)

def check_default_options(options):
  """Add sensible defaults to options when not defined"""
  # TODO: Add ability to read from a configuration file
  if "pageSize" not in options:
    options["pageSize"] = 100
  
  return options

def endpoint_url(folder, endpoint, id="", modifier="", config_file='servicepytan_config.json', tenant_id=""):
  """Constructs API request URL based on key parameters

  Retrives JSON response from provided URL with a number of parameters to customize the request.

  Args:
      folder: A string based on the endpoint groupings
      endpoint: A string indicating the endpoint you want to address.
      id: A string for the id of the endpoint object you're addressing.
      modifier: A string to modify the url to address the additional endpoint.
      config_file: A string for the file path to the configuration file.
      tenant_id: A string to manually adjust the tenant id.

  Returns:
      A URL String

  Raises:
      TBD
  """  
  # Adds ability to manually switch up the Tenant ID for apps that have multiples
  if tenant_id == "":
    tenant_id = get_tenant_id(config_file)

  url = f"{URL_ROOT}/{folder}/v2/tenant/{tenant_id}/{endpoint}"
  if id != "": url = f"{url}/{id}"
  if modifier != "": url = f"{url}/{modifier}"
  return url
    
def create_credential_file(name="servicepytan_config.json"):
  """Creates and saves an unfilled configuration file.

  Args:
      name: A string path to the credentials file

  Returns:
      A filepath string
  """  
  file = open(name, 'w')
  file.write(
    """{
    "CLIENT_ID": "",
    "CLIENT_SECRET": "",
    "APP_ID": "",
    "APP_KEY": "",
    "TENANT_ID": ""
    }"""
  )
  file.close()
  return name

def get_timezone_by_file(config_file='servicepytan_config.json'):
  """Retrieves timezone from the configuration file.

  Args:
      config_file: A string path to the credentials file

  Returns:
      Timezone string
  """    
  # Read File
  f = open(config_file)
  config = json.load(f)
  if "TIMEZONE" in config:
    timezone = config['TIMEZONE']
  else:
    timezone = ""
  f.close()
  return timezone


from servicepytan.utils import request_json, get_timezone_by_file, endpoint_url

class Report:
  """Primary class for retrieving Reporting Endpoint Data.

  Attributes:
      config_file: a string file path to the config file.
  """
  def __init__(self, category, report_id, config_file="servicepytan_config.json"):
    """Inits DataService with configuration file and authentication settings."""
    self.config_file = config_file
    self.timezone = get_timezone_by_file(config_file)
    self.category = category
    self.report_id = report_id

  def get_data(self, params):
    """get report data"""
    url = endpoint_url("reporting",f"report-category/{self.category}/reports/{self.report_id}/data", config_file=self.config_file)
    return request_json(url, json_payload=params, config_file=self.config_file, request_type="POST")


from servicepytan.data import DataService

def get_booked_jobs_by_agent(start_date, end_date, config_file="servicepytan_config.json"):
  booked_jobs = DataService(config_file=config_file).get_jobs_created_between(start_date, end_date)
  employees = DataService(config_file=config_file).get_employees()
  pass


"""Authenticating with ServiceTitan API"""

import requests
import json
from servicepytan import URL_ROOT, AUTH_ROOT

def get_auth_token(client_id, client_secret):
  """Fetches Auth Token.

  Retrieves authentication token for completing a request against the API

  Args:
      client_id: String, provided from the integration settings
      client_secret: String, provided from the integration settings

  Returns:
      Authentication token

  Raises:
      TBD
  """

  url = f"{AUTH_ROOT}/connect/token"

  querystring = {"Content-Type":"application/x-www-form-urlencoded"}

  payload = f"grant_type=client_credentials&client_id={client_id}&client_secret={client_secret}"
  headers = {"Content-Type": "application/x-www-form-urlencoded"}

  response = requests.request("POST", url, data=payload, headers=headers, params=querystring)

  return json.loads(response.text)

def get_auth_token_by_file(config_file='servicepytan_config.json'):
  """Fetches Auth Token using the config_file.

  Retrives the CLIENT_ID and CLIENT_SECRET entries from config_file.

  Args:
      config_file: String, path to the config file defaults to 'servicepytan_config.json'

  Returns:
      Authentication token

  Raises:
      TBD
  """
  # Read File
  f = open(config_file)
  creds = json.load(f)
  client_id = creds['CLIENT_ID']
  client_secret = creds['CLIENT_SECRET']
  f.close()
  return get_auth_token(client_id, client_secret)["access_token"]

def get_app_key(config_file='servicepytan_config.json'):
  """Fetches App Key from the config_file.

  Retrives the APP_KEY entry from config_file.

  Args:
      config_file: String, path to the config file defaults to 'servicepytan_config.json'

  Returns:
      App Key

  Raises:
      TBD
  """
  f = open(config_file)
  creds = json.load(f)
  app_key = creds['APP_KEY']
  f.close()
  return app_key

def get_tenant_id(config_file='servicepytan_config.json'):
  """Fetches Tenant ID from the config_file.

  Retrives the TENANT_ID entry from config_file.

  Args:
      config_file: String, path to the config file defaults to 'servicepytan_config.json'

  Returns:
      Tenant ID

  Raises:
      TBD
  """
  f = open(config_file)
  creds = json.load(f)
  tenant_id = creds['TENANT_ID']
  f.close()
  return tenant_id 

def get_auth_headers(config_file='servicepytan_config.json'):
  """Generates the Authentication Headers for each API request

  Creates an object that includes the auth token and app key formatted to create the auth headers.

  Args:
      config_file: String, path to the config file defaults to 'servicepytan_config.json'

  Returns:
      Object

  Raises:
      TBD
  """
  return {
      "Authorization": get_auth_token_by_file(config_file),
      "ST-App-Key": get_app_key(config_file)
  }